import 'package:aitrainer_app/bloc/customer_change/customer_change_bloc.dart';
import 'package:aitrainer_app/localization/app_localization.dart';
import 'package:aitrainer_app/repository/customer_repository.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

// ignore: must_be_immutable
class CustomerBodyTypePage extends StatefulWidget {
  _CustomerBodyTypePageState _state;

  _CustomerBodyTypePageState createState() {
    _state = _CustomerBodyTypePageState();
    return _state;
  }
}

class BodyTypeItem {
  static String endomorph = "endomorph";
  static String ectomorph = "ectomorph";
  static String mesomorph = "mesomorph";
}

class _CustomerBodyTypePageState extends State<CustomerBodyTypePage> {
  String selected;
  @override
  Widget build(BuildContext context) {
    final CustomerRepository customerRepository =
        ModalRoute.of(context).settings.arguments;
    final double cWidth = MediaQuery.of(context).size.width * 0.75;

    return Scaffold(
        appBar: AppBar(
          title: Row(
            mainAxisAlignment: MainAxisAlignment.end,
            children: <Widget>[
              Image.asset(
                'asset/image/WT_long_logo.png',
                fit: BoxFit.cover,
                height: 65.0,
              ),
            ],
          ),
          backgroundColor: Colors.transparent,
        ),
        body: Container(
          decoration: BoxDecoration(
            image: DecorationImage(
              image: AssetImage('asset/image/WT_light_background.png'),
              fit: BoxFit.cover,
              alignment: Alignment.center,
            ),
          ),
          child: BlocProvider(
              create: (context) =>
                  CustomerChangeBloc(customerRepository: customerRepository),
              child: Builder(builder: (context) {
                // ignore: close_sinks
                CustomerChangeBloc changeBloc =
                    BlocProvider.of<CustomerChangeBloc>(context);

                return Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    Divider(),
                    Wrap(
                        //runAlignment: WrapAlignment.center,
                        alignment: WrapAlignment.center,
                        children: [
                          Text(
                            AppLocalizations.of(context)
                                .translate("Your Body Type"),
                            textAlign: TextAlign.center,
                            style: TextStyle(
                                color: Colors.orange,
                                fontSize: 42,
                                fontFamily: 'Arial',
                                fontWeight: FontWeight.w900),
                          )
                        ]),
                    Divider(),
                    FlatButton(
                        child: Container(
                            width: cWidth,
                            child: Column(
                              children: [
                                Text(
                                    AppLocalizations.of(context)
                                        .translate("Endomorph"),
                                    textWidthBasis: TextWidthBasis.longestLine,
                                    style: TextStyle(
                                        color: Colors.blue,
                                        fontSize: 32,
                                        fontFamily: 'Arial',
                                        fontWeight: FontWeight.w900)),
                              ],
                            )),
                        padding: EdgeInsets.all(10.0),
                        shape: getShape(
                            customerRepository, BodyTypeItem.endomorph),
                        onPressed: () => {
                              setState(() {
                                selected = BodyTypeItem.endomorph;
                                changeBloc.add(CustomerBodyTypeChange(bodyType: selected));
                                print(selected);
                              }),
                            }),
                    Divider(),
                    FlatButton(
                        child: Container(
                          width: cWidth,
                          child: Column(
                            children: [
                              InkWell(
                                child: Text(
                                  AppLocalizations.of(context)
                                      .translate("Ectomorph"),
                                  style: TextStyle(
                                      color: Colors.blue,
                                      fontSize: 32,
                                      fontFamily: 'Arial',
                                      fontWeight: FontWeight.w900),
                                ),
                                highlightColor: Colors.white,
                              ),
                            ],
                          ),
                        ),
                        padding: EdgeInsets.all(10.0),
                        shape: getShape(customerRepository, BodyTypeItem.ectomorph ),

                        onPressed: () => {
                              setState(() {
                                selected = BodyTypeItem.ectomorph;
                                changeBloc.add(CustomerBodyTypeChange(bodyType: selected));
                                print(selected);
                              }),
                            }),
                    Divider(),
                    FlatButton(
                        child: Container(
                          width: cWidth,
                          child: Column(
                            children: [
                              InkWell(
                                child: Text(
                                  AppLocalizations.of(context)
                                      .translate("Mesomorph"),
                                  style: TextStyle(
                                      color: Colors.blue,
                                      fontSize: 32,
                                      fontFamily: 'Arial',
                                      fontWeight: FontWeight.w900),
                                ),
                                highlightColor: Colors.white,
                              ),
                            ],
                          ),
                        ),
                        padding: EdgeInsets.all(10.0),
                        shape: getShape(customerRepository, BodyTypeItem.mesomorph ),
                        onPressed: () => {
                              setState(() {
                                selected = BodyTypeItem.mesomorph;
                                changeBloc.add(CustomerBodyTypeChange(bodyType: selected));

                                print(selected);
                              }),
                            }),
                    Divider(),
                    RaisedButton(
                      color: Colors.orange,
                      textColor: Colors.white,
                      child: InkWell(
                          child: Text(
                              AppLocalizations.of(context).translate("Next"))),
                      onPressed: () => {
                        changeBloc.add(CustomerSave()),
                        Navigator.of(context).pop(),
                        Navigator.of(context).pushNamed("customerWelcomePage", arguments: customerRepository)
                      },
                    )
                  ],
                );
              })),
        ));
  }

  dynamic getShape(CustomerRepository customerRepository, String fitnessLevel) {
    String selected = customerRepository.bodyType;
    dynamic returnCode = (selected == fitnessLevel)
        ? RoundedRectangleBorder(
            side: BorderSide(width: 4, color: Colors.orange),
          )
        : RoundedRectangleBorder(
            side: BorderSide(width: 1, color: Colors.blue),
          );
    //return
    return returnCode;
  }
}
